"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Playlist = exports.MasterPlaylist = exports.isEncryptedChunk = exports.isNormalChunk = exports.isInitialChunk = exports.M3U8ParseError = void 0;
const common_1 = require("../utils/common");
const log_1 = __importDefault(require("../utils/log"));
class M3U8ParseError extends Error {
}
exports.M3U8ParseError = M3U8ParseError;
const isInitialChunk = (chunk) => {
    return chunk.isInitialChunk;
};
exports.isInitialChunk = isInitialChunk;
const isNormalChunk = (chunk) => {
    return !(0, exports.isInitialChunk)(chunk);
};
exports.isNormalChunk = isNormalChunk;
const isEncryptedChunk = (chunk) => {
    return chunk.isEncrypted;
};
exports.isEncryptedChunk = isEncryptedChunk;
const getTagBody = (line) => line.split(":").slice(1).join(":");
const parseTagBody = (body) => {
    const matchResult = body.match(/([^=,]+)(=([^",]|(".+?"))*)?/g);
    const result = {};
    if (matchResult.length > 0) {
        for (const match of matchResult) {
            const [key, ...value] = match.split("=");
            const valueStr = value.join("=");
            result[key] = valueStr.startsWith('"') ? valueStr.slice(1, valueStr.length - 1) : valueStr;
        }
    }
    return result;
};
class MasterPlaylist {
    constructor({ m3u8Content, m3u8Url }) {
        this.streams = [];
        this.m3u8Content = m3u8Content;
        this.m3u8Url = m3u8Url;
        this.parse();
    }
    parse() {
        const lines = this.m3u8Content.split("\n");
        for (let i = 0; i <= lines.length - 1; i++) {
            /**
             * v8 引擎内部对 split/slice 出的字符串有一个对 parent 的引用
             * 并且默认不会被 GC 当 parent string 很长时会造成内存泄漏
             * 此处复制了一次字符串避免此情况
             * See also: https://github.com/nodejs/help/issues/711
             */
            const currentLine = lines[i].split("").join("").trim();
            if (currentLine.startsWith("#EXT-X-STREAM-INF")) {
                // stream information
                const nextLine = lines[i + 1];
                if (!nextLine) {
                    throw new M3U8ParseError("Invalid M3U8 file.");
                }
                const tagBody = getTagBody(currentLine);
                const parsedTagBody = parseTagBody(tagBody);
                if (!parsedTagBody["BANDWIDTH"]) {
                    /**
                     * @see https://datatracker.ietf.org/doc/html/rfc8216#section-4.3.4.2
                     * Every EXT-X-STREAM-INF tag MUST include the BANDWIDTH attribute.
                     */
                    throw new M3U8ParseError("Missing BANDWIDTH attribute for streams.");
                }
                if (!nextLine.startsWith("http") && !this.m3u8Url) {
                    throw new M3U8ParseError("Missing full url for M3U8.");
                }
                const url = (0, common_1.buildFullUrl)(this.m3u8Url, nextLine);
                const streamInfo = Object.assign(Object.assign({ url, bandwidth: +parsedTagBody["BANDWIDTH"] }, (parsedTagBody["CODECS"] ? { codecs: parsedTagBody["CODECS"] } : {})), (parsedTagBody["FRAME-RATE"] ? { frameRate: +parsedTagBody["FRAME-RATE"] } : {}));
                if (parsedTagBody["RESOLUTION"] && parsedTagBody["RESOLUTION"].includes("x")) {
                    const [x, y] = parsedTagBody["RESOLUTION"].split("x").map((n) => parseInt(n));
                    streamInfo.resolution = {
                        width: x,
                        height: y,
                    };
                }
                this.streams.push(streamInfo);
            }
            // TODO: Support #EXT-X-MEDIA
        }
    }
}
exports.MasterPlaylist = MasterPlaylist;
class Playlist {
    constructor({ m3u8Content, m3u8Url = "" }) {
        this.sequenceId = 0;
        this.isEnd = false;
        this.chunks = [];
        this.encryptKeys = [];
        this.averageChunkLength = 0;
        this.totalChunkLength = 0;
        this.m3u8Content = m3u8Content;
        this.m3u8Url = m3u8Url;
        this.parse();
    }
    /**
     * 解析基本属性
     */
    parse() {
        let key, iv, isEncrypted = false, warned = false;
        const lines = this.m3u8Content.split("\n");
        for (let i = 0; i <= lines.length - 1; i++) {
            /**
             * v8 引擎内部对 split/slice 出的字符串有一个对 parent 的引用
             * 并且默认不会被 GC 当 parent string 很长时会造成内存泄漏
             * 此处复制了一次字符串避免此情况
             * See also: https://github.com/nodejs/help/issues/711
             */
            const currentLine = lines[i].split("").join("").trim();
            if (currentLine.startsWith("#EXT-X-MEDIA-SEQUENCE")) {
                /**
                 * @see https://datatracker.ietf.org/doc/html/rfc8216#section-4.3.3.2
                 */
                const tagBody = getTagBody(currentLine);
                this.sequenceId = parseInt(tagBody);
            }
            if (currentLine.startsWith("#EXT-X-KEY")) {
                /**
                 * @see https://datatracker.ietf.org/doc/html/rfc8216#section-4.3.2.4
                 */
                const tagBody = getTagBody(currentLine);
                const parsedTagBody = parseTagBody(tagBody);
                if (parsedTagBody["METHOD"] === "AES-128") {
                    isEncrypted = true;
                    key = parsedTagBody["URI"];
                    if (parsedTagBody["IV"]) {
                        iv = parsedTagBody["IV"].match(/0x([^,]+)/)[1];
                    }
                    this.encryptKeys.push(key);
                }
                else if (parsedTagBody["METHOD"] === "NONE") {
                    isEncrypted = false;
                }
                else {
                    isEncrypted = false;
                    if (!warned) {
                        log_1.default.warning(`Unsupported encryption method: "${parsedTagBody["METHOD"]}". Chunks will not be decrypted.`);
                        warned = true;
                    }
                }
            }
            if (currentLine.startsWith("#EXT-X-MAP")) {
                /**
                 * Initial segment
                 * @see https://datatracker.ietf.org/doc/html/rfc8216#section-4.3.2.5
                 */
                const tagBody = getTagBody(currentLine);
                const parsedTagBody = parseTagBody(tagBody);
                const initialSegmentUrl = parsedTagBody["URI"];
                if (!initialSegmentUrl) {
                    throw new M3U8ParseError("Missing URL for initialization segment");
                }
                if (!initialSegmentUrl.startsWith("http") && !this.m3u8Url) {
                    throw new M3U8ParseError("Missing full url for M3U8.");
                }
                if (isEncrypted && !iv) {
                    /**
                     * If the Media Initialization Section declared by an EXT-X-MAP tag is
                     * encrypted with a METHOD of AES-128, the IV attribute of the EXT-X-KEY
                     * tag that applies to the EXT-X-MAP is REQUIRED.
                     * @see https://datatracker.ietf.org/doc/html/rfc8216#section-4.3.2.5
                     */
                    throw new M3U8ParseError("Missing IV for encrypted initialization segment");
                }
                this.chunks.push({
                    url: (0, common_1.buildFullUrl)(this.m3u8Url, initialSegmentUrl),
                    isEncrypted,
                    key,
                    iv,
                    isInitialChunk: true,
                });
            }
            if (currentLine.startsWith("#EXT-X-ENDLIST")) {
                this.isEnd = true;
                break;
            }
            if (currentLine.startsWith("#EXTINF")) {
                const tagBody = getTagBody(currentLine);
                const chunkLength = parseFloat(tagBody.split(",")[0]) || 5.0;
                let nextLine, j = i;
                while (j <= lines.length - 1) {
                    j += 1;
                    if (!lines[j].startsWith("#")) {
                        nextLine = lines[j];
                        break;
                    }
                }
                if (!nextLine) {
                    throw new M3U8ParseError("Invalid M3U8 file.");
                }
                if (!nextLine.startsWith("http") && !this.m3u8Url) {
                    throw new M3U8ParseError("Missing full url for M3U8.");
                }
                if (isEncrypted) {
                    this.chunks.push({
                        url: (0, common_1.buildFullUrl)(this.m3u8Url, nextLine),
                        length: chunkLength,
                        isEncrypted: true,
                        key,
                        iv,
                        sequenceId: this.sequenceId,
                        isInitialChunk: false,
                    });
                }
                else {
                    this.chunks.push({
                        url: (0, common_1.buildFullUrl)(this.m3u8Url, nextLine),
                        length: chunkLength,
                        isEncrypted: false,
                        sequenceId: this.sequenceId,
                        isInitialChunk: false,
                    });
                }
                /**
                 * @see https://datatracker.ietf.org/doc/html/rfc8216#section-3
                 * The Media Sequence Number of the first segment in the Media Playlist is either 0 or declared in the * Playlist (Section 4.3.3.2). The Media Sequence Number of every other segment is equal to the Media * Sequence Number of the segment that precedes it plus one.
                 */
                this.sequenceId++;
            }
        }
    }
    /**
     * average length of chunks
     * @returns
     */
    getChunkLength() {
        if (this.averageChunkLength) {
            return this.averageChunkLength;
        }
        const totalLength = this.chunks.filter(exports.isNormalChunk).reduce((acc, cur) => acc + cur.length, 0);
        const totalCount = this.chunks.filter(exports.isNormalChunk).length;
        const result = totalLength / totalCount;
        this.averageChunkLength = result;
        return result;
    }
    getTotalChunkLength() {
        if (this.totalChunkLength) {
            return this.totalChunkLength;
        }
        const result = this.chunks.filter(exports.isNormalChunk).reduce((acc, cur) => acc + cur.length, 0);
        this.totalChunkLength = result;
        return result;
    }
}
exports.Playlist = Playlist;
class M3U8 {
    constructor({ m3u8Content, m3u8Url, initPrimaryKey, }) {
        this.m3u8Content = m3u8Content;
        this.m3u8Url = m3u8Url;
        this.initPrimaryKey = initPrimaryKey;
    }
    parse() {
        if (this.m3u8Content.includes("#EXT-X-STREAM-INF")) {
            return new MasterPlaylist({ m3u8Content: this.m3u8Content, m3u8Url: this.m3u8Url });
        }
        else {
            return new Playlist({
                m3u8Content: this.m3u8Content,
                m3u8Url: this.m3u8Url,
            });
        }
    }
}
exports.default = M3U8;
//# sourceMappingURL=m3u8.js.map