"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const https_proxy_agent_1 = require("https-proxy-agent");
const socks_proxy_agent_1 = require("socks-proxy-agent");
const log_1 = __importDefault(require("./log"));
const winreg_1 = __importDefault(require("winreg"));
const readRegistryKey = (key) => {
    return new Promise((resolve, reject) => {
        key.values((err, items) => {
            if (err) {
                reject(err);
            }
            resolve(items);
        });
    });
};
class InvalidProxyServerError extends Error {
}
class ProxyAgentHelper {
    constructor() {
        this.proxyAgentInstance = null;
    }
    /**
     * Set up proxy server and initialize the proxy agent instance
     * @param proxy
     * @param params
     * @param params.allowNonPrefixSocksProxy Treat proxy server without protocol as socks5 proxy for backward compatibility
     */
    setProxy(proxy, { allowNonPrefixSocksProxy = false } = {}) {
        if (!proxy) {
            return;
        }
        if (proxy.startsWith("http://") || proxy.startsWith("https://")) {
            // HTTP Proxy
            this.proxyAgentInstance = new https_proxy_agent_1.HttpsProxyAgent(proxy, {
                keepAlive: true,
            });
            log_1.default.debug(`HTTP/HTTPS Proxy set: ${proxy}`);
        }
        else if (proxy.startsWith("socks")) {
            if (proxy.startsWith("socks4")) {
                throw new InvalidProxyServerError("Socks4 is not supported. Please use HTTP or Socks5 proxy.");
            }
            // Socks5 Proxy
            try {
                const [_, host, port] = proxy.match(/socks5?(?:(?<=5)h)?[:：]\/\/(.+)[:：](\d+)/);
                this.proxyAgentInstance = new socks_proxy_agent_1.SocksProxyAgent(`socks5h://${host}:${port}`, {
                    keepAlive: true,
                });
                log_1.default.debug(`Socks5 Proxy set: socks5h://${host}:${port}`);
            }
            catch (e) {
                throw new InvalidProxyServerError("Proxy server invalid.");
            }
        }
        else if (allowNonPrefixSocksProxy && !proxy.match(/\//)) {
            // For compatibility, use proxy without protocol as socks5 proxy
            try {
                const [_, host, port] = proxy.match(/(.+)[:：](\d+)/);
                this.proxyAgentInstance = new socks_proxy_agent_1.SocksProxyAgent(`socks5h://${host}:${port}`, {
                    keepAlive: true,
                });
                log_1.default.debug(`Socks5 Proxy set: socks5h://${host}:${port}`);
            }
            catch (e) {
                throw new InvalidProxyServerError("Proxy server invalid.");
            }
        }
        else {
            throw new InvalidProxyServerError("Proxy server invalid.");
        }
    }
    getProxyAgentInstance() {
        return this.proxyAgentInstance;
    }
    /**
     * Read proxy configuration from environment variables.
     * By default, ALL_PROXY, HTTP_PROXY and HTTPS_PROXY will be used.
     * Note: environment variables will override system proxy in Windows.
     */
    readProxyConfigurationFromEnv() {
        const proxySettings = process.env.ALL_PROXY || process.env.HTTP_PROXY || process.env.HTTPS_PROXY;
        if (proxySettings) {
            this.setProxy(process.env.ALL_PROXY || process.env.HTTP_PROXY || process.env.HTTPS_PROXY);
        }
    }
    /**
     * Read Windows system proxy from registry
     */
    readWindowsSystemProxy() {
        return __awaiter(this, void 0, void 0, function* () {
            if (process.platform !== "win32") {
                // not a windows environment
                return;
            }
            const key = new winreg_1.default({
                hive: winreg_1.default.HKCU,
                key: "\\Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
            });
            try {
                const items = yield readRegistryKey(key);
                const proxyEnableItem = items.find((item) => item.name === "ProxyEnable");
                const proxyServerItem = items.find((item) => item.name === "ProxyServer");
                const isProxyEnable = proxyEnableItem.value === "0x1";
                if (isProxyEnable && proxyServerItem && proxyServerItem.value !== "") {
                    this.setProxy(proxyServerItem.value);
                }
            }
            catch (_a) {
                // ignore
            }
        });
    }
}
exports.default = new ProxyAgentHelper();
//# sourceMappingURL=agent.js.map